import { QueryBuilder } from "knex";


interface QUser {
    uid : string
};

export type GroupSet = Set<string>;


/**
 * There are 6 levels of authorisation for a group
 *  none : can't even know the group exists
 *  viewer : can see the group
 *  memebr : part of the group
 *  speaker : allowed to speak for the group
 *  admin : admin of the group
 *  supervisor : allowed to take control of the group
 * 
 * Levels 0 and 5 are a bit special. For levels 1 - 4, the functions are as follows
 */

// These functions return the list of all groups that the query user (quser) has corresponding privilege for

export async function forViewer(quser : QUser) : Promise<GroupSet>{
    throw new Error("Not implemented");
}

export async function forMember(quser : QUser) : Promise<GroupSet>{
    throw new Error("Not implemented");
}

export async function forSpeaker(quser : QUser) : Promise<GroupSet>{
    throw new Error("Not implemented");
}

export async function forAdmin(quser : QUser) : Promise<GroupSet>{
    throw new Error("Not implemented");
}

// These functions test if the quser has corresponding privileges for the given group

export async function isViewer(quser : QUser, gid : string) : Promise<boolean>{
    let groupSet = await forViewer(quser);
    return groupSet.has(gid);
}

export async function isMember(quser : QUser, gid : string) : Promise<boolean>{
    let groupSet = await forMember(quser);
    return groupSet.has(gid);
}

export async function isSpeaker(quser : QUser, gid : string) : Promise<boolean>{
    let groupSet = await forSpeaker(quser);
    return groupSet.has(gid);
}

export async function isAdmin(quser : QUser, gid : string) : Promise<boolean>{
    let groupSet = await forAdmin(quser);
    return groupSet.has(gid);
}

export async function isSupervisor(quser : QUser, gid : string) : Promise<boolean>{
    throw new Error('Not implemented');
};